@@ -38,7 +38,7 @@ pub trait Indexable {
 
 pub trait IndexableFile: Indexable {
     /// Should read from the current position until the end of the line, omitting the \n
-    fn read_current_line(&mut self, buf: &mut Vec<u8>) -> Result<usize>;
+    fn read_current_line(&mut self, buf: &mut Vec<u8>, line: usize) -> Result<usize>;
 
     /// Should seek the file to the given line `line`
     fn seek_line(&mut self, line: usize) -> Result<()>;
@@ -59,16 +59,15 @@ pub trait ReadByLine: IndexableFile {
     /// Reads the given line
     fn read_line(&mut self, line: usize) -> Result<String> {
         self.seek_line(line)?;
-
         let mut read_data = Vec::new();
-        self.read_current_line(&mut read_data)?;
+        self.read_current_line(&mut read_data, line)?;
         Ok(String::from_utf8(read_data)?)
     }
 
     /// Reads the given line and stores into `buf`
     fn read_line_raw(&mut self, line: usize, buf: &mut Vec<u8>) -> Result<usize> {
         self.seek_line(line)?;
-        Ok(self.read_current_line(buf)?)
+        Ok(self.read_current_line(buf, line)?)
     }
 
     /// Do a binary search on `ReadByLine` implementing Types, since it provides everything required
@@ -106,19 +105,60 @@ pub trait ReadByLine: IndexableFile {
 
         Err(error::Error::NotFound)
     }
+
+    /// Do a binary search on `ReadByLine` implementing Types, since it provides everything required
+    /// for binary search. Only works with sorted files
+    #[inline]
+    fn binary_search_raw(&mut self, x: &[u8]) -> Result<usize> {
+        self.binary_search_raw_by(|p| p.cmp(x))
+    }
+
+    /// Do a binary search by on `ReadByLine` implementing Types, since it provides everything required
+    /// for binary search. Only works with sorted files
+    fn binary_search_raw_by<F>(&mut self, mut f: F) -> Result<usize>
+    where
+        F: FnMut(&[u8]) -> std::cmp::Ordering,
+    {
+        let mut size = self.total_lines();
+        let mut left = 0;
+        let mut right = size;
+
+        let mut buf = Vec::new();
+
+        while left < right {
+            let mid = left + size / 2;
+
+            buf.clear();
+            self.read_line_raw(mid, &mut buf)?;
+            let cmp = f(&buf);
+
+            if cmp == Ordering::Less {
+                left = mid + 1;
+            } else if cmp == Ordering::Greater {
+                right = mid;
+            } else {
+                return Ok(mid);
+            }
+
+            size = right - left;
+        }
+
+        Err(error::Error::NotFound)
+    }
 }
 
 #[cfg(test)]
 mod tests {
     use rand::{distributions::Uniform, Rng};
 
-    use crate::string::IndexedString;
+    use crate::{any::IndexedReader, string::IndexedString};
 
     use super::*;
     use std::{
         convert::TryInto,
         fs::read_to_string,
         io::{prelude::*, BufReader},
+        sync::Arc,
     };
 
     #[test]
@@ -129,7 +169,7 @@ mod tests {
 
     #[test]
     fn test() {
-        let input_files = &["input1", "LICENSE"];
+        let input_files = &["simple", "LICENSE", "input1"];
 
         for input_file in input_files {
             let file = format!("./testfiles/{}", input_file);
@@ -137,10 +177,10 @@ mod tests {
             // Test File
             let mut indexed_file = File::open_raw(&file).expect("failed opening indexed file");
             test_reader(&mut indexed_file, &file);
-            assert_eq!(
-                indexed_file.read_all().unwrap(),
-                read_to_string(&file).unwrap()
-            );
+
+            let mut buf = Vec::new();
+            indexed_file.read_all(&mut buf).unwrap();
+            assert_eq!(buf, read_to_string(&file).unwrap().as_bytes());
 
             // Test IndexedString
             let file_content = read_to_string(&file).unwrap();
@@ -152,6 +192,12 @@ mod tests {
             let indexed_str_file: Result<IndexedString> = indexed_file.try_into();
             assert!(indexed_str_file.is_ok());
             test_reader(&mut indexed_str_file.unwrap(), &file);
+
+            // Test File to indexed Vec<u8>
+            let indexed_file = File::open_raw(&file).expect("failed opening indexed file");
+            let indexed_str_file: Result<IndexedReader<Vec<u8>>> = indexed_file.try_into();
+            assert!(indexed_str_file.is_ok());
+            test_reader(&mut indexed_str_file.unwrap(), &file);
         }
     }
 
@@ -169,10 +215,15 @@ mod tests {
             let read = reader.read_line(line);
 
             assert!(read.is_ok());
-            assert_eq!(original, read.unwrap());
+
+            let mut read = read.unwrap();
+            read.pop();
+
+            assert_eq!(original, read);
 
             let mut buf = Vec::new();
             let res = reader.read_line_raw(line, &mut buf);
+            buf.pop();
             assert!(res.is_ok());
             assert_eq!(original, String::from_utf8(buf).unwrap());
         }
@@ -192,12 +243,53 @@ mod tests {
             let read = reader.read_line(line);
 
             assert!(read.is_ok());
-            assert_eq!(*original, read.unwrap());
+
+            let mut read = read.unwrap();
+            read.pop();
+
+            assert_eq!(*original, read);
 
             let mut buf = Vec::new();
             let res = reader.read_line_raw(line, &mut buf);
+            buf.pop();
             assert!(res.is_ok());
             assert_eq!(*original, String::from_utf8(buf).unwrap());
         }
     }
+
+    #[test]
+    fn test_no_new_line() {
+        let index = Index::new(vec![0, 5, 10, 15, 20, 25, 30, 35, 40, 45]);
+        let text = "this is a text with fifty characters. this is a te";
+
+        let mut indexed_text = IndexedString::new_custom(text, Arc::new(index.zero_len()));
+
+        for i in 0..indexed_text.total_lines() {
+            let read = indexed_text.read_line(i).unwrap();
+            assert_eq!(read.len(), 5);
+        }
+
+        let mut buf = Vec::new();
+        indexed_text.reader.read_all(&mut buf).unwrap();
+        assert_eq!(buf, text.as_bytes());
+    }
+
+    #[test]
+    fn test_write_to() {
+        let file = "./testfiles/pre_indexed";
+
+        let mut indexed_data = Vec::new();
+        File::open(&file)
+            .expect("failed opening pre_indexed file")
+            .write_to(&mut indexed_data)
+            .unwrap();
+
+        let mut raw_data = Vec::new();
+        std::fs::File::open(file)
+            .unwrap()
+            .read_to_end(&mut raw_data)
+            .unwrap();
+
+        assert_eq!(raw_data, indexed_data);
+    }
 }
